@@ -585,7 +585,7 @@ static __latent_entropy int dup_mmap(struct mm_struct *mm,
 
 			vma_get_file(tmp);
 			i_mmap_lock_write(mapping);
-			if (tmp->vm_flags & VM_SHARED)
+			if (vma_is_shared_maywrite(tmp))
 				mapping_allow_writable(mapping);
 			flush_dcache_mmap_lock(mapping);
 			/* insert tmp into the share list, just after mpnt */
